
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "getfdt.h"

#include <libfdt.h>

unsigned long fdt_get_ulong(const char *vpath, const char *pname,
		unsigned long defvalue)
{
	int fd, size, node;
	void *fdtbuf;
	const void *prop;
	unsigned long retval = defvalue;

	fd = open("/dev/fdtexport", O_RDONLY);
	if (fd < 0) {
		fprintf(stderr, "Can't open /dev/fdtexport: %s\n",
				strerror(errno));
		return defvalue;
	}

	size = lseek(fd, 0, SEEK_END);
	if (size < 0) {
		fprintf(stderr, "Can't get size to mmap: %s\n",
				strerror(errno));
		goto out_close;
	}

	fdtbuf = mmap(0, size, PROT_READ, MAP_SHARED, fd, 0);
	if (fdtbuf == MAP_FAILED) {
		fprintf(stderr, "Can't mmap: %s\n", strerror(errno));
		goto out_close;
	}

	node = fdt_path_offset(fdtbuf, vpath);
	if (node < 0) {
		fprintf(stderr, "Can't find node '%s'\n", vpath);
		goto out_munmap;
	}

	prop = fdt_getprop(fdtbuf, node, pname, NULL);
	if (prop == NULL) {
		fprintf(stderr, "Can't find property '%s'\n", pname);
		goto out_munmap;
	}
	
	retval = fdt32_to_cpu(*((unsigned long int*)prop));

out_munmap:
	munmap(fdtbuf, size);

out_close:
	close(fd);

	return retval;
}

